unit ItemDownload;

interface

uses
	SysUtils, Classes, ComCtrls, Controls, Forms, IdHTTP,
	{HTTPApp,} YofUtils, IdGlobal, IdException, IdComponent, IniFiles, {DateUtils,}
	GikoSystem, BoardGroup, MonaUtils;

type
	TDownloadItem = class;
	TGikoDownloadType = (gdtBoard, gdtThread);
	TGikoDownloadState = (gdsWait, gdsWork, gdsComplete, gdsDiffComplete, gdsNotModify, gdsAbort, gdsError);
	TGikoCgiStatus = (gcsOK, gcsINCR, gcsERR);
	TGikoDLProgress = (gdpStd, gdpAll, gdpDatOchi, gdpOfflaw);

	TGikoWorkEvent = procedure(Sender: TObject; AWorkMode: TWorkMode; const AWorkCount: Integer; ID: Integer) of object;
	TGikoWorkBeginEvent = procedure(Sender: TObject; AWorkMode: TWorkMode; const AWorkCountMax: Integer; ID: Integer) of object;
	TGikoWorkEndEvent = procedure(Sender: TObject; AWorkMode: TWorkMode; ID: Integer) of object;
	TDownloadEndEvent = procedure(Sender: TObject; Item: TDownloadItem) of object;
	TDownloadMsgEvent = procedure(Sender: TObject; Item: TDownloadItem; Msg: string; Icon: TGikoMessageIcon) of object;

	TCgiStatus = record
		FStatus: TGikoCgiStatus;
		FSize: Integer;
		FErrText: string;
	end;


	TDownloadThread = class(TThread)
	private
		FIndy: TIdHttp;
		FItem: TDownloadItem;
		FNumber: Integer;
		FAbort: Boolean;
		FMsg: string;
		FIcon: TGikoMessageIcon;
		FSessionID: string;
		FOnWork: TGikoWorkEvent;
		FOnWorkBegin: TGikoWorkBeginEvent;
		FOnWorkEnd: TGikoWorkEndEvent;
		FOnDownloadEnd: TDownloadEndEvent;
		FOnDownloadMsg: TDownloadMsgEvent;

		procedure FireDownloadEnd;
		procedure FireDownloadMsg;
		procedure GetSessionID;
		procedure WorkBegin(Sender: TObject; AWorkMode: TWorkMode; const AWorkCountMax: Integer);
		procedure WorkEnd(Sender: TObject; AWorkMode: TWorkMode);
		procedure Work(Sender: TObject; AWorkMode: TWorkMode; const AWorkCount: Integer);
		function ParseCgiStatus(Content: string): TCgiStatus;
		function DatDownload(ItemType: TGikoDownloadType; URL: string; Modified: TDateTime; RangeStart: Integer; AdjustLen: Integer): Boolean;
		function DeleteStatusLine(Content: string): string;
	protected
		procedure Execute; override;
	public
		property Item: TDownloadItem read FItem write FItem;
		property Number: Integer read FNumber write FNumber;
		constructor Create(CreateSuspended: Boolean);
		destructor Destroy; override;
		procedure Abort;
		property OnWork: TGikoWorkEvent read FOnWork write FOnWork;
		property OnWorkBegin: TGikoWorkBeginEvent read FOnWorkBegin write FOnWorkBegin;
		property OnWorkEnd: TGikoWorkEndEvent read FOnWorkEnd write FOnWorkEnd;
		property OnDownloadEnd: TDownloadEndEvent read FOnDownloadEnd write FOnDownloadEnd;
		property OnDownloadMsg: TDownloadMsgEvent read FOnDownloadMsg write FOnDownloadMsg;
	end;

	TDownloadItem = class(TObject)
	private
		FDownType: TGikoDownloadType;
		FBoard: TBoard;
		FThreadItem: TThreadItem;

		FContentLength: Integer;
		FLastModified: TDateTime;
		FContent: string;
		FResponseCode: Smallint;
		FState: TGikoDownloadState;
		FErrText: string;
	public
		procedure SaveListFile;
		procedure SaveItemFile;

		property DownType: TGikoDownloadType read FDownType write FDownType;
		property Board: TBoard read FBoard write FBoard;
		property ThreadItem: TThreadItem read FThreadItem write FThreadItem;

		property ContentLength: Integer read FContentLength write FContentLength;
		property LastModified: TDateTime read FLastModified write FLastModified;
		property Content: string read FContent write FContent;
		property ResponseCode: Smallint read FResponseCode write FResponseCode;
		property State: TGikoDownloadState read FState write FState;
		property ErrText: string read FErrText write FErrText;
	end;

implementation

constructor TDownloadThread.Create(CreateSuspended: Boolean);
begin
	inherited Create(CreateSuspended);
	FIndy := TIdHttp.Create(nil);

	FIndy.OnWorkBegin := WorkBegin;
	FIndy.OnWorkEnd := WorkEnd;
	FIndy.OnWork := Work;
end;

destructor TDownloadThread.Destroy;
begin
	FIndy.Free;
	inherited;
end;

function RFC1123_Date(aDate : TDateTime) : String;
const
	 StrWeekDay : String = 'MonTueWedThuFriSatSun';
	 StrMonth   : String = 'JanFebMarAprMayJunJulAugSepOctNovDec';
var
	 Year, Month, Day       : Word;
	 Hour, Min,   Sec, MSec : Word;
	 DayOfWeek              : Word;
begin
	 DecodeDate(aDate, Year, Month, Day);
	 DecodeTime(aDate, Hour, Min,   Sec, MSec);
	 DayOfWeek := ((Trunc(aDate) - 2) mod 7);
	 Result := Copy(StrWeekDay, 1 + DayOfWeek * 3, 3) + ', ' +
						 Format('%2.2d %s %4.4d %2.2d:%2.2d:%2.2d',
										[Day, Copy(StrMonth, 1 + 3 * (Month - 1), 3),
										 Year, Hour, Min, Sec]);
end;

procedure TDownloadThread.Execute;
var
	ResStream: TMemoryStream;

	URL: string;
	CgiStatus: TCgiStatus;
	Modified: TDateTime;
	RangeStart: Integer;
	AdjustLen: Integer;
	Idx: Integer;
	ATitle: string;
	DownloadResult: Boolean;
	Abone: Boolean;
begin
	while not Terminated do begin
		FAbort := False;
		FIndy.Request.CustomHeaders.Clear;
		FIndy.Response.Clear;
		FIndy.Request.Clear;
		FIndy.Request.UserAgent := GikoSys.GetUserAgent;
		FIndy.RecvBufferSize := Gikosys.Setting.RecvBufferSize;
		FIndy.ProxyParams.BasicAuthentication := False;
		{$IFDEF DEBUG}
		Writeln('------------------------------------------------------------');
		{$ENDIF}
		//FIndy.AllowCookies := False;
		if GikoSys.Setting.ReadProxy then begin
			if GikoSys.Setting.ProxyProtocol then
				FIndy.ProtocolVersion := pv1_1
			else
				FIndy.ProtocolVersion := pv1_0;
			FIndy.ProxyParams.ProxyServer := GikoSys.Setting.ReadProxyAddress;
			FIndy.ProxyParams.ProxyPort := GikoSys.Setting.ReadProxyPort;
			FIndy.ProxyParams.ProxyUsername := GikoSys.Setting.ReadProxyUserID;
			FIndy.ProxyParams.ProxyPassword := GikoSys.Setting.ReadProxyPassword;
			if GikoSys.Setting.ReadProxyUserID <> '' then
				FIndy.ProxyParams.BasicAuthentication := True;
			{$IFDEF DEBUG}
			Writeln('vLVݒ肠');
			Writeln('zXg: ' + GikoSys.Setting.ReadProxyAddress);
			Writeln('|[g: ' + IntToStr(GikoSys.Setting.ReadProxyPort));
			{$ENDIF}
		end else begin
			if GikoSys.Setting.Protocol then
				FIndy.ProtocolVersion := pv1_1
			else
				FIndy.ProtocolVersion := pv1_0;
			FIndy.ProxyParams.ProxyServer := '';
			FIndy.ProxyParams.ProxyPort := 80;
			FIndy.ProxyParams.ProxyUsername := '';
			FIndy.ProxyParams.ProxyPassword := '';
			{$IFDEF DEBUG}
			Writeln('vLVݒȂ');
			{$ENDIF}
		end;

		FIndy.Request.ContentRangeStart := 0;
		FIndy.Request.LastModified := ZERO_DATE;
		ResStream := TMemoryStream.Create;
		try
			try
				//********************
				//DAT or Subject擾
				//********************
				Item.ResponseCode := 0;
				RangeStart := 0;
				AdjustLen := 0;
				Modified := 0;
				if Item.DownType = gdtBoard then begin
					{$IFDEF DEBUG}
					Writeln('Subject擾');
					Writeln('URL: ' + Item.Board.GetReadCgiURL);
					Writeln('Modified: ' + FloatToStr(Item.Board.LastModified));
					{$ENDIF}
					URL := Item.Board.GetReadCgiURL;
					Modified := Item.Board.LastModified;
				end else if Item.DownType = gdtThread then begin
					{$IFDEF DEBUG}
					Writeln('DAT擾');
					Writeln('URL: ' + Item.ThreadItem.GetDatURL);
					Writeln('Modified: ' + FloatToStr(Item.ThreadItem.LastModified));
					{$ENDIF}
					URL := Item.ThreadItem.GetDatURL;
					Modified := Item.ThreadItem.LastModified;
					if Item.ThreadItem.Size > 0 then begin
						{$IFDEF DEBUG}
						Writeln('RangeStart: ' + IntToStr(Item.ThreadItem.Size));
						{$ENDIF}
						//ځ[`FbN̂߂PoCgO擾
						RangeStart := Item.ThreadItem.Size;
						AdjustLen := -1;
					end;
				end;
				Abone := False;
				DownloadResult := DatDownload(Item.DownType, URL, Modified, RangeStart, AdjustLen);
				{$IFDEF DEBUG}
				Writeln('ResponseCode: ' + IntToStr(FIndy.ResponseCode));
				{$ENDIF}
				if Item.DownType = gdtThread then begin
					if Item.ResponseCode = 416 then begin
						Abone := True;
						DownloadResult := True;
					end else if DownloadResult and (AdjustLen = -1) and (Item.Content[1] <> #10) then
						Abone := True;
				end;

				if Trim(FIndy.Response.RawHeaders.Values['Date']) <> '' then begin
					if Item.DownType = gdtBoard then
						Item.Board.LastGetTime := MonaUtils.DateStrToDateTime(FIndy.Response.RawHeaders.Values['Date'])
					else
						Item.ThreadItem.ParentBoard.LastGetTime := MonaUtils.DateStrToDateTime(FIndy.Response.RawHeaders.Values['Date']);
				end;

				if DownloadResult then begin
					{$IFDEF DEBUG}
					Writeln('Date:' + FIndy.Response.RawHeaders.Values['Date']);
					{$ENDIF}
					if Abone then begin
						{$IFDEF DEBUG}
						Writeln('ځ[񌟏o');
						{$ENDIF}
						ATitle := Item.ThreadItem.Title;
						if ATitle = '' then
							ATitle := 'i̕sj';
						//擾PoCgڂLFłȂꍇ́uځ[vĂ邩̂ōĎ擾
						RangeStart := 0;
						AdjustLen := 0;
						FMsg := 'uځ[vôōĎ擾s܂ - [' + ATitle + ']';
						FIcon := gmiWhat;
						if Assigned(OnDownloadMsg) then
							Synchronize(FireDownloadMsg);
						if not DatDownload(Item.DownType, URL, ZERO_DATE, RangeStart, AdjustLen) then
							Item.State := gdsError;
						{$IFDEF DEBUG}
						Writeln('ځ[Ď擾');
						Writeln('ResponseCode: ' + IntToStr(Item.ResponseCode));
						{$ENDIF}
					end else if (Item.DownType = gdtThread) and (AdjustLen = -1) and (Item.Content[1] = #10) then begin
						//擾PoCgڂLF̏ꍇi擾j͓LF폜
						Item.Content := Copy(Item.Content, 2, Length(Item.Content));
					end;
				end else begin
					Item.State := gdsError;
					if (Item.DownType = gdtBoard) and (Item.ResponseCode = 302) then begin
						FMsg := 'ړ]Ȃ̂ŔXVsĂ';
						FIcon := gmiNG;
						if Assigned(OnDownloadMsg) then
							Synchronize(FireDownloadMsg);
					end;
				end;

				//********************
				//dat.gz擾(1)
				//********************
				if (Item.DownType = gdtThread) and (Item.ResponseCode = 302) then begin
					{$IFDEF DEBUG}
					Writeln('dat.gz擾');
					{$ENDIF}
					ATitle := Item.ThreadItem.Title;
					if ATitle = '' then
						ATitle := 'i̕sj';
					FMsg := 'dat݂Ȃ߉ߋO(dat.gz)T܂ - [' + ATitle + ']';
					FIcon := gmiWhat;
					if Assigned(OnDownloadMsg) then
						Synchronize(FireDownloadMsg);
					URL := Item.ThreadItem.GetDatgzURL;
					Modified := Item.ThreadItem.LastModified;
					RangeStart := 0;
					AdjustLen := 0;
					if not DatDownload(Item.DownType, URL, Modified, RangeStart, AdjustLen) then
						Item.State := gdsError;
					{$IFDEF DEBUG}
					Writeln('ResponseCode: ' + IntToStr(Item.ResponseCode));
					{$ENDIF}
				end;

				//********************
				//dat.gz擾(2)
				//********************
{
				if (Item.DownType = gdtThread) and (Item.ResponseCode = 302) then begin
					ATitle := Item.ThreadItem.Title;
					if ATitle = '' then
						ATitle := 'i̕sj';
					FMsg := 'ߋO(1)݂Ȃ߉ߋO(2)T܂ - [' + ATitle + ']';
					FIcon := gmiWhat;
					if Assigned(OnDownloadMsg) then
						Synchronize(FireDownloadMsg);
					URL := Item.ThreadItem.GetOldDatgzURL;
					Modified := Item.ThreadItem.LastModified;
					RangeStart := 0;
					AdjustLen := 0;
					if not DatDownload(Item.DownType, URL, Modified, RangeStart, AdjustLen) then
						Item.State := gdsError;
				end;
}

				//********************
				//offlaw.cgiŎ擾
				//********************
				FSessionID := '';
				Synchronize(GetSessionID);
				if (Item.DownType = gdtThread) and (Item.ResponseCode = 302) and (FSessionID <> '') then begin
					{$IFDEF DEBUG}
					Writeln('offlaw.cgiŎ擾');
					{$ENDIF}
					ATitle := Item.ThreadItem.Title;
					if ATitle = '' then
						ATitle := 'i̕sj';
					FMsg := 'dat.gz݂Ȃofflaw.cgi𗘗p܂ - [' + ATitle + ']';
					FIcon := gmiWhat;
					if Assigned(OnDownloadMsg) then
						Synchronize(FireDownloadMsg);
					URL := Item.ThreadItem.GetOfflawCgiURL(FSessionID);
					Modified := Item.ThreadItem.LastModified;
					RangeStart := 0;
					AdjustLen := 0;
					if not DatDownload(Item.DownType, URL, Modified, RangeStart, AdjustLen) then begin
						{$IFDEF DEBUG}
						Writeln('ResponseCode: ' + IntToStr(Item.ResponseCode));
						{$ENDIF}
						Item.State := gdsError;

						if (Item.DownType = gdtThread) and (Item.ResponseCode = 302) then begin
							FMsg := 'ړ]Ȃ̂ŔXVsĂB';
							FIcon := gmiNG;
							if Assigned(OnDownloadMsg) then
								Synchronize(FireDownloadMsg);
						end;

					end else begin
						CgiStatus := ParseCgiStatus(Item.Content);
						{$IFDEF DEBUG}
						Writeln('ResponseCode: ' + IntToStr(Item.ResponseCode));
						{$ENDIF}
						case CgiStatus.FStatus of
							gcsOK: begin
								{$IFDEF DEBUG}
								Writeln('CGIStatus: OK');
								{$ENDIF}
								Item.ResponseCode := 200;
								Item.Content := DeleteStatusLine(Item.Content);
								Item.ContentLength := CgiStatus.FSize;
							end;
							gcsINCR: begin
								//͂肦Ȃ
								{$IFDEF DEBUG}
								Writeln('CGIStatus: 206');
								{$ENDIF}
								Item.ResponseCode := 206;
								Item.Content := DeleteStatusLine(Item.Content);
								Item.ContentLength := CgiStatus.FSize;
							end;
							gcsERR: begin
								{$IFDEF DEBUG}
								Writeln('CGIStatus: 404(ERROR)');
								{$ENDIF}
								Item.ResponseCode := 404;
								Item.State := gdsError;
								Item.ErrText := CgiStatus.FErrText;
							end;
						end;
						if (Item.ResponseCode = 404) and (AnsiPos('ߋOqɂŔ', Item.ErrText) <> 0) then begin
							{$IFDEF DEBUG}
							Writeln('ߋO擾');
							{$ENDIF}
							ATitle := Item.ThreadItem.Title;
							if ATitle = '' then
								ATitle := 'i̕sj';
							FMsg := 'ߋOqɂŔ - [' + ATitle + ']';
							FIcon := gmiWhat;
							if Assigned(OnDownloadMsg) then
								Synchronize(FireDownloadMsg);
							Idx := Pos(' ', Item.ErrText);
							if Idx <> 0 then begin
								URL := Copy(Item.ErrText, Idx + 1, Length(Item.ErrText));
								if Pos('../', URL) = 1 then
									URL := Copy(URL, 4, Length(URL));
								URL := GikoSys.UrlToServer(Item.ThreadItem.ParentBoard.URL) + URL;
								Modified := Item.ThreadItem.LastModified;
								RangeStart := 0;
								AdjustLen := 0;
								if not DatDownload(Item.DownType, URL, Modified, RangeStart, AdjustLen) then
									Item.State := gdsError;
								{$IFDEF DEBUG}
								Writeln('ResponseCode: ' + IntToStr(Item.ResponseCode));
								{$ENDIF}
							end;
						end;
					end;
				end else begin
					if (Item.DownType = gdtThread) and (Item.ResponseCode = 302) and (FSessionID = '') then begin
						{$IFDEF DEBUG}
						Writeln('OCĂȂ̂ŉߋO擾s');
						{$ENDIF}
						ATitle := Item.ThreadItem.Title;
						if ATitle = '' then
							ATitle := 'i̕sj';
						FMsg := 'OCĂȂߒTƂo܂ - [' + ATitle + ']';
						FIcon := gmiSAD;
						if Assigned(OnDownloadMsg) then
							Synchronize(FireDownloadMsg);
					end;
				end;

				case Item.ResponseCode of
					200: Item.State := gdsComplete;
					206: Item.State := gdsDiffComplete;
					304: Item.State := gdsNotModify;
					else
						Item.State := gdsError;
				end;
{
				//ƎvǁBBB
				if (Item.ResponseCode in [200, 206]) and (Item.Content = '') then
					Item.State := gdsError;
				Item.LastModified := FIndy.Response.LastModified;
				//擾łPoCgOƂĂƂ̓}CiX
				Item.ContentLength := FIndy.Response.ContentLength + AdjustLen;
				try
					ResStream.Clear;
					FIndy.Get(URL, ResStream);
					Item.Content := GikoSys.GzipDecompress(ResStream, FIndy.Response.ContentEncoding);
					if (Item.DownType = gdtThread) and (AdjustLen = -1) and (Item.Content[1] <> #10) then begin
						//擾PoCgڂLFłȂꍇ́uځ[vĂ邩̂ōĎ擾
						//ŃbZ[W\Cxg
						//event
						FMsg := 'uځ[vôōĎ擾s܂';
						if Assigned(OnDownloadMsg) then
							Synchronize(FireDownloadMsg);
						FIndy.Request.ContentRangeStart := 0;
						FIndy.Request.ContentRangeEnd := 0;
						AdjustLen := 0;
						ResStream.Clear;
						FIndy.Get(URL, ResStream);
						Item.Content := GikoSys.GzipDecompress(ResStream, FIndy.Response.ContentEncoding);
					end else if (Item.DownType = gdtThread) and (AdjustLen = -1) and (Item.Content[1] = #10) then begin
						//擾PoCgڂLF̏ꍇi擾j͓LF폜
						Item.Content := Copy(Item.Content, 2, Length(Item.Content));
					end;
				except
					Item.State := gdsError;
				end;
				Item.ResponseCode := FIndy.ResponseCode;
}
{
				try
					ResStream.Clear;
					FIndy.Get(URL, ResStream);
					Item.Content := GikoSys.GzipDecompress(ResStream, FIndy.Response.ContentEncoding);
				except
					Item.State := gdsError;
				end;

				CgiStatus := ParseCgiStatus(Item.Content);
				if CgiStatus.FStatus = gcsOK then begin
					if CgiStatus.FSize = 0 then
						Item.State := gdsNotModify
					else if Item.DownType = gdtBoard then
						Item.State := gdsComplete
					else
						Item.State := gdsDiffComplete;
				end else if CgiStatus.FStatus = gcsINCR then begin
					Item.State := gdsComplete;
				end else if CgiStatus.FStatus = gcsERR then begin
					Item.State := gdsError;
					Item.ErrText := CgiStatus.FErrText;
				end;
				Item.ContentLength := CgiStatus.FSize;
				}
			except
				Item.State := gdsError;
			end;
			//Item.ResponseCode := FIndy.ResponseCode;
			if FAbort then
				Item.State := gdsAbort;
		finally
			if Assigned(OnDownloadEnd) then
				Synchronize(FireDownloadEnd);
			ResStream.Free;
		end;
		if Terminated then Break;
		Suspend;
	end;
end;

function TDownloadThread.DatDownload(ItemType: TGikoDownloadType; URL: string; Modified: TDateTime; RangeStart: Integer; AdjustLen: Integer): Boolean;
var
	ResponseCode: Integer;
	ResStream: TMemoryStream;
begin
	ResponseCode := -1;
	if (ItemType = gdtThread) and (RangeStart > 0) then begin
//	if (ItemType = gdtThread) and (Item.ThreadItem.Size > 0) then begin
//		FIndy.Request.ContentRangeStart := Item.ThreadItem.Size + AdjustLen;
		FIndy.Request.ContentRangeStart := RangeStart + AdjustLen;
		FIndy.Request.ContentRangeEnd := 0;
	end else begin
		FIndy.Request.ContentRangeStart := 0;
		FIndy.Request.ContentRangeEnd := 0;
	end;

	FIndy.Request.CustomHeaders.Clear;
	FIndy.Request.CacheControl := 'no-cache';
	FIndy.Request.CustomHeaders.Add('Pragma: no-cache');
	if (Modified <> 0) and (Modified <> ZERO_DATE) then begin
		FIndy.Request.LastModified := modified - OffsetFromUTC;
		//FIndy.Request.CustomHeaders.Add('If-Modified-Since: ' + RFC1123_Date(modified - OffsetFromUTC) + ' GMT');
	end;
//	FIndy.Request.AcceptEncoding := 'gzip';
	if RangeStart = 0 then
		FIndy.Request.AcceptEncoding := 'gzip'
	else
		FIndy.Request.AcceptEncoding := '';
	ResStream := TMemoryStream.Create;
	try
		try
			ResStream.Clear;
			{$IFDEF DEBUG}
			Writeln('URL: ' + URL);
			{$ENDIF}
			FIndy.Get(URL, ResStream);
			Item.Content := GikoSys.GzipDecompress(ResStream, FIndy.Response.ContentEncoding);
			Item.LastModified := FIndy.Response.LastModified;
			//擾łPoCgOƂĂƂ̓}CiX
//			Item.ContentLength := FIndy.Response.ContentLength + AdjustLen;
			Item.ContentLength := Length(Item.Content) + AdjustLen;
			//ƎvǁBBB
//			if (FIndy.ResponseCode in [200, 206]) and (Item.Content = '') then
//				Result := False
			if Item.Content = '' then
				Result := False
			else
				Result := True;
			{$IFDEF DEBUG}
			Writeln('擾ŗOȂ');
			{$ENDIF}
			ResponseCode := FIndy.ResponseCode;
		except
			on E: EIdSocketError do begin
				Item.Content := '';
				Item.LastModified := ZERO_DATE;
				Item.ContentLength := 0;
				Item.ErrText := E.Message;
				Result := False;
				ResponseCode := -1;
			end;
			on E: EIdConnectException do begin
				Item.Content := '';
				Item.LastModified := ZERO_DATE;
				Item.ContentLength := 0;
				Item.ErrText := E.Message;
				Result := False;
				ResponseCode := -1;
			end;
			on E: Exception do begin
				{$IFDEF DEBUG}
				Writeln('擾ŗO');
				Writeln('E.Message: ' + E.Message);
				{$ENDIF}
				Item.Content := '';
				Item.LastModified := ZERO_DATE;
				Item.ContentLength := 0;
				Item.ErrText := E.Message;
				ResponseCode := FIndy.ResponseCode;
				Result := False;
			end;
		end;
	finally
		if (Item.ContentLength = 0) and (ResponseCode = 206) then
			Item.ResponseCode := 304
		else
			Item.ResponseCode := ResponseCode;
		ResStream.Free;
	end;
end;

procedure TDownloadThread.FireDownloadEnd;
begin
	OnDownloadEnd(self, Item);
end;

procedure TDownloadThread.FireDownloadMsg;
begin
	OnDownloadMsg(Self, Item, FMsg, FIcon);
end;

procedure TDownloadThread.GetSessionID;
begin
	FSessionID := GikoSys.Dolib.SessionID;
end;

procedure TDownloadThread.Abort;
begin
	FAbort := True;
	FIndy.DisconnectSocket;
end;

procedure TDownloadThread.WorkBegin(Sender: TObject; AWorkMode: TWorkMode; const AWorkCountMax: Integer);
begin
	if Assigned(OnWorkBegin) then
		OnWorkBegin(Sender, AWorkMode, AWorkCountMax, FNumber);
end;

procedure TDownloadThread.WorkEnd(Sender: TObject; AWorkMode: TWorkMode);
begin
	if Assigned(OnWorkEnd) then
		OnWorkEnd(Sender, AWorkMode, FNumber);
end;

procedure TDownloadThread.Work(Sender: TObject; AWorkMode: TWorkMode; const AWorkCount: Integer);
begin
	if Assigned(OnWork) then
		OnWork(Sender, AWorkMode, AWorkCount, FNumber);
end;

function TDownloadThread.ParseCgiStatus(Content: string): TCgiStatus;
var
	StatusLine: string;
	Line: string;
	Idx: Integer;
	Status: string;
	Size: string;
	Msg: string;
begin
// [+OK] ̏ꍇ͍
// [-INCR] (Incorrect)̏ꍇׂ͂Ẵf[^
// [-ERR (eLXg)]̏ꍇ͂Ȃ񂩃G[
// F+OK 23094/512K
//     -INCR 23094/512K
//     -ERR ȔȂł
	Idx := AnsiPos(#10, Content);
	StatusLine := Copy(Content, 0, Idx);

	Idx := AnsiPos(' ', Content);
	Status := Copy(StatusLine, 0, Idx - 1);
	Line := Copy(StatusLine, Idx + 1, Length(StatusLine));

//	Idx := AnsiPos('/', Line);
	if Trim(Status) = '-ERR' then
		Msg := Trim(Line)
	else
		Size := Copy(Line, 0, Idx - 1);

	if Trim(Status) = '+OK' then
		Result.FStatus := gcsOK
	else if Trim(Status) = '-INCR' then
		Result.FStatus := gcsINCR
	else if Trim(Status) = '-ERR' then begin
		Result.FStatus := gcsERR;
		Result.FErrText := Msg;
		Result.FSize := 0;
		Exit;
	end else begin
		{$IFDEF DEBUG}
		Writeln(Status);
		{$ENDIF}
		Result.FStatus := gcsERR;
		Result.FErrText := 'G[Ȃ񂾂ǁA悭ȂG[';
		Result.FSize := 0;
		Exit;
	end;

	if GikoSys.IsNumeric(Size) then
		Result.FSize := StrToInt(Size)
	else begin
		Result.FSize := 0;
		Result.FStatus := gcsERR;
		Result.FErrText := 'Xe[^X͎s[' + StatusLine + ']';
	end;
end;

//蔲ȏ1sڂ
function TDownloadThread.DeleteStatusLine(Content: string): string;
var
	SList: TStringList;
begin
	SList := TStringList.Create;
	try
		SList.Text := Content;
		if SList.Count > 1 then
			SList.Delete(0);
		Result := SList.Text;
	finally
		SList.Free;
	end;
end;

procedure TDownloadItem.SaveListFile;
var
	i: Integer;
	index: Integer;
	NewItem: TThreadItem;
	NewList: TList;
//	SaveCount: Integer;
	NumCount: Integer;
	Body: TStringList;
	Rec: TSubjectRec;
begin
	NewList := TList.Create;
	Body := TStringList.Create;
	try
		//_E[hݒi[Jj
		Board.RoundDate := Now;
		//T[ot@C̍XVݒ
		Board.LastModified := LastModified;

		//VXg쐬
		//VXgɌÂXg̃OȂ炻VXgɒǉ
		//ÂOȂ΁AVɃXIuWFNg쐬
		Body.Text := Content;
		NumCount := 0;
		for i := 0 to Body.Count - 1 do begin
			//if i = 0 then Continue;	//Psڂ̓Xe[^XŝߏȂ

			Rec := GikoSys.DivideSubject(Body[i]);
			Rec.FFileName := Trim(Rec.FFileName);
			if (Rec.FTitle = '') and (Rec.FCount = 0) then Continue;
			Inc(NumCount);
			index := Board.GetIndex(Rec.FFileName);
			if index = -1 then begin
				//VXbh
				NewItem := TThreadItem.Create;
				NewItem.FileName := Rec.FFileName;
				NewItem.Title := Rec.FTitle;
//				NewItem.Count := Rec.FCount;
				NewItem.AllResCount := Rec.FCount;
				NewItem.ParentBoard := Board;
				NewItem.No := NumCount;
				NewItem.RoundDate := ZERO_DATE;
				NewItem.LastModified := ZERO_DATE;
				NewItem.AgeSage := gasNew;
				NewList.Add(NewItem);
			end else begin
				//Board.Items[index].Count := Count;
				//Board.Items[index].Count := Rec.FCount;
				if Board.Items[index].No > NumCount then
					Board.Items[index].AgeSage := gasAge
				else if Board.Items[index].AllResCount < Rec.FCount then
					Board.Items[index].AgeSage := gasSage
				else
					Board.Items[index].AgeSage := gasNone;


				Board.Items[index].No := NumCount;
				Board.Items[index].AllResCount := Rec.FCount;
//				if not Board.Items[index].IsLogFile then
//					Board.Items[index].Count := Rec.FCount;
				NewList.Add(Board.Items[index]);
				Board.DeleteList(index);
			end;
		end;

		//VXgɖACeVXgɒǉ
		for i := 0 to Board.Count - 1 do begin
			if Board.Items[i].IsLogFile then begin
				inc(NumCount);
				Board.Items[i].No := NumCount;
				Board.Items[i].AllResCount := Board.Items[i].Count;
				Board.Items[i].NewResCount := 0;
				Board.Items[i].AgeSage := gasNone;
				NewList.Add(Board.Items[i]);
			end;
		end;

		//ÂXgiXĝ݁BXIuWFNĝ͏Ȃj
		for i := Board.Count - 1 downto 0 do
			Board.DeleteList(i);

		//VXg{[hIuWFNgɒǉ
		for i := 0 to NewList.Count - 1 do
			Board.Add(TThreadItem(NewList[i]));

		//Xg(subject.txt)ۑ
//		GikoSys.ForceDirectoriesEx(GikoSys.GetLogDir + Board.BBSID);
//		Body.SaveToFile(GikoSys.GetSubjectFileName(Board.BBSID));
		GikoSys.ForceDirectoriesEx(ExtractFilePath(Board.GetSubjectFileName));
		Body.SaveToFile(Board.GetSubjectFileName);
	finally
		Body.Free;
		NewList.Free;
	end;
end;

{procedure TDownloadItem.SaveListFile;
var
	i: Integer;
	index: Integer;
	NewItem: TThreadItem;
	NewList: TList;
//	SaveCount: Integer;
	NumCount: Integer;
	Body: TStringList;
	Rec: TSubjectRec;
begin
	NewList := TList.Create;
	Body := TStringList.Create;
	try
		//ݒ
		Board.RoundDate := Now;
		//T[ot@C̍XVݒ
		Board.LastModified := LastModified;

		//Xgۑ擾
		//SaveCount := MaxInt;

		//ÂXg烍OACe폜
		for i := Board.Count - 1 downto 0 do
			if not Board.Items[i].IsLogFile then
				Board.Delete(i);

		//VXg쐬
		//VXgɌÂXg̃OȂ炻VXgɒǉ
		//ÂOȂ΁AVɃXIuWFNg쐬
		Body.Text := Content;
//		Loop := Min(Body.Count, SaveCount);
		NumCount := 0;
//		for i := 0 to Loop - 1 do begin
		for i := 0 to Body.Count - 1 do begin
			if i = 0 then Continue;	//Psڂ̓Xe[^XŝߏȂ

			Rec := GikoSys.DivideSubject(Body[i]);
			if (Rec.FTitle = '') and (Rec.FCount = 0) then Continue;
			Inc(NumCount);
			index := Board.GetIndex(Rec.FFileName);
			if index = -1 then begin
				NewItem := TThreadItem.Create;
				NewItem.FileName := Rec.FFileName;
				NewItem.Title := Rec.FTitle;
				NewItem.Count := Rec.FCount;
				NewItem.ParentBoard := Board;
				NewItem.No := NumCount;
				NewItem.RoundDate := ZERO_DATE;
				NewItem.LastModified := ZERO_DATE;
				NewList.Add(NewItem);
			end else begin
				//Board.Items[index].Count := Count;
				Board.Items[index].No := NumCount;
				NewList.Add(Board.Items[index]);
				Board.DeleteList(index);
			end;
		end;

		//VXgɖÂOLACeVXgɒǉ
		for i := 0 to Board.Count - 1 do begin
			inc(NumCount);
			Board.Items[i].No := NumCount;
			NewList.Add(Board.Items[i]);
		end;

		//ÂXgiXĝ݁BXIuWFNĝ͏Ȃj
		for i := Board.Count - 1 downto 0 do
			Board.DeleteList(i);

		//VXg{[hIuWFNgɒǉ
		for i := 0 to NewList.Count - 1 do
			Board.Add(TThreadItem(NewList[i]));

		//Xg(subject.txt)ۑ
//		GikoSys.ForceDirectoriesEx(GikoSys.GetLogDir + Board.BBSID);
//		Body.SaveToFile(GikoSys.GetSubjectFileName(Board.BBSID));
		GikoSys.ForceDirectoriesEx(ExtractFilePath(Board.GetSubjectFileName));
		Body.SaveToFile(Board.GetSubjectFileName);
	finally
		Body.Free;
		NewList.Free;
	end;
end;
}
procedure TDownloadItem.SaveItemFile;
var
	Body: TStringList;
	Cnt: Integer;
	OldCnt: Integer;
	FileName: string;
	ini: TMemIniFile;
	Res: TResRec;
	NewRes: Integer;
begin
	if Trim(Content) = '' then
		Exit;
	FileName := ThreadItem.GetThreadFileName;
	GikoSys.ForceDirectoriesEx(ExtractFilePath(FileName));

//	Cnt := 0;
	Body := TStringList.Create;
	try
//		if FileExists(FileName) and (ResponseCode = 206) then begin
		if FileExists(FileName) and (State = gdsDiffComplete) then begin
//			Body.Text := Content;
//			if Body.Count > 0 then
//				Body.Delete(0);
//			Content := Body.Text;
			Body.LoadFromFile(FileName);
			OldCnt := Body.Count;
			Body.Text := Body.Text + Content;
			Body.SaveToFile(FileName);
			NewRes := Body.Count - OldCnt;
			Cnt := Body.Count;
		end else begin
			Body.Text := Content;
//			if Body.Count > 0 then
//				Body.Delete(0);
			Body.SaveToFile(FileName);

			if ThreadItem.Title = '' then begin
				Res := GikoSys.DivideStrLine(Body[0]);
				ThreadItem.Title := Res.FTitle;
			end;
			ThreadItem.Size := 0;
			//ThreadItem.Count := 0;
			ThreadItem.AllResCount := 0;
			ThreadItem.NewResCount := 0;
			OldCnt := 0;
			NewRes := Body.Count;
			Cnt := Body.Count;
		end;
		Cnt := Body.Count;
	finally
		Body.Free;
	end;
	ThreadItem.RoundDate := Now;
	ThreadItem.Size := ThreadItem.Size + ContentLength;
	ThreadItem.LastModified := LastModified;
	ThreadItem.Count := Cnt;
	ThreadItem.AllResCount := Cnt;
	ThreadItem.NewResCount := NewRes;
	ThreadItem.IsLogFile := True;
	ThreadItem.NewReceive := OldCnt + 1;
	ThreadItem.UnRead := True;
	ThreadItem.ParentBoard.UnRead := ThreadItem.ParentBoard.UnRead + 1;
//	if ThreadItem.RoundNo = 6 then
//		ThreadItem.RoundNo := 0;

	//ُI̓CfbNXXVȂ߁Ae|쐬B
	//Iɂ͍폜
	//ُÍANɃe|čXV
	ini := TMemIniFile.Create(ChangeFileExt(FileName, '.tmp'));
	try
		ini.WriteDateTime('Setting', 'RoundDate', ThreadItem.RoundDate);
		ini.WriteDateTime('Setting', 'LastModified', ThreadItem.LastModified);
		ini.WriteInteger('Setting', 'Size', ThreadItem.Size);
		ini.WriteInteger('Setting', 'Count', ThreadItem.Count);
		ini.WriteInteger('Setting', 'AllResCount', ThreadItem.AllResCount);
		ini.WriteInteger('Setting', 'NewResCount', ThreadItem.NewResCount);
		ini.WriteInteger('Setting', 'NewReceive', ThreadItem.NewReceive);
//		ini.WriteInteger('Setting', 'RoundNo', ThreadItem.RoundNo);
		ini.WriteBool('Setting', 'Round', ThreadItem.Round);
		ini.WriteBool('Setting', 'UnRead', ThreadItem.UnRead);
		ini.UpdateFile;
	finally
		ini.Free;
	end;
end;

end.
